//
//  Utils.m
//  NUIdemo
//
//  Created by zhouguangdong on 2019/12/26.
//  Copyright © 2019 Alibaba idst. All rights reserved.
//

#import <Foundation/Foundation.h>
#import "NuiSdkUtils.h"
#include <netdb.h>
#include <arpa/inet.h>
#import <AdSupport/ASIdentifierManager.h>
#import "token/AccessToken.h"

@implementation NuiSdkUtils
//Get Document Dir
-(NSString *)dirDoc {
    NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
    NSString *documentsDirectory = [paths objectAtIndex:0];
    TLog(@"app_home_doc: %@",documentsDirectory);
    return documentsDirectory;
}

//create dir for saving files
-(NSString *)createDir {
    NSString *documentsPath = [self dirDoc];
    NSFileManager *fileManager = [NSFileManager defaultManager];
    NSString *testDirectory = [documentsPath stringByAppendingPathComponent:@"voices"];
    // 创建目录
    BOOL res=[fileManager createDirectoryAtPath:testDirectory withIntermediateDirectories:YES attributes:nil error:nil];
    if (res) {
        TLog(@"文件夹创建成功");
    }else
        TLog(@"文件夹创建失败");
    return testDirectory;
}

-(void) getTicket:(NSMutableDictionary*) dictM Type:(TokenTicketType)type{
    //郑重提示:
    //  语音交互服务需要先准备好账号，并开通相关服务。具体步骤请查看：
    //    https://help.aliyun.com/zh/isi/getting-started/start-here
    //
    //原始账号:
    //  账号(子账号)信息主要包括AccessKey ID(后续简称为ak_id)和AccessKey Secret(后续简称为ak_secret)。
    //  此账号信息一定不可存储在app代码中或移动端侧，以防账号信息泄露造成资费损失。
    //
    //STS临时凭证:
    //  由于账号信息下发给客户端存在泄露的可能，阿里云提供的一种临时访问权限管理服务STS(Security Token Service)。
    //  STS是由账号信息ak_id和ak_secret，通过请求生成临时的sts_ak_id/sts_ak_secret/sts_token
    //  (为了区别原始账号信息和STS临时凭证, 命名前缀sts_表示STS生成的临时凭证信息)
    //什么是STS：https://help.aliyun.com/zh/ram/product-overview/what-is-sts
    //STS SDK概览：https://help.aliyun.com/zh/ram/developer-reference/sts-sdk-overview
    //STS Python SDK调用示例：https://help.aliyun.com/zh/ram/developer-reference/use-the-sts-openapi-example
    //
    //账号需求说明:
    //  若使用离线功能(离线语音合成、唤醒), 则必须app_key、ak_id和ak_secret，或app_key、sts_ak_id、sts_ak_secret和sts_token
    //  若使用在线功能(语音合成、实时转写、一句话识别、录音文件转写等), 则只需app_key和token

    //项目创建
    //  创建appkey请查看：https://help.aliyun.com/zh/isi/getting-started/start-here
    NSString *APPKEY = @"<您申请创建的app_key>";
    [dictM setObject:APPKEY forKey:@"app_key"]; // 必填
    
    self.curTokenTicketType = type;
    self.curAppkey = APPKEY;

    if (type == get_sts_access_from_server_for_online_features) {
        //方法一，仅适合在线语音交互服务(强烈推荐):
        //  客户远端服务端使用STS服务获得STS临时凭证，然后下发给移动端侧，详情请查看：
        //    https://help.aliyun.com/document_detail/466615.html 使用其中方案二使用STS获取临时账号。
        //  然后在移动端侧通过AccessToken()获得Token和有效期，用于在线语音交互服务。
        NSString *STS_AK_ID = @"STS.<服务器生成的具有时效性的临时凭证>";
        NSString *STS_AK_SECRET = @"<服务器生成的具有时效性的临时凭证>";
        NSString *STS_TOKEN = @"<服务器生成的具有时效性的临时凭证>";
        NSString *TOKEN = @"<由STS生成的临时访问令牌>";
        TOKEN = [self generateToken:STS_AK_ID withSecret:STS_AK_SECRET withStsToken:STS_TOKEN];
        if (TOKEN == NULL) {
            NSLog(@"generate token failed");
            return;
        }
        [dictM setObject:TOKEN forKey:@"token"]; // 必填
        self.curToken = TOKEN;
    } else if (type == get_sts_access_from_server_for_offline_features) {
        //方法二，仅适合离线语音交互服务(强烈推荐):
        //  客户远端服务端使用STS服务获得STS临时凭证，然后下发给移动端侧，详情请查看：
        //    https://help.aliyun.com/document_detail/466615.html 使用其中方案二使用STS获取临时账号。
        NSString *STS_AK_ID = @"STS.<服务器生成的具有时效性的临时凭证>";
        NSString *STS_AK_SECRET = @"<服务器生成的具有时效性的临时凭证>";
        NSString *STS_TOKEN = @"<服务器生成的具有时效性的临时凭证>";
        [dictM setObject:STS_AK_ID forKey:@"ak_id"]; // 必填
        [dictM setObject:STS_AK_SECRET forKey:@"ak_secret"]; // 必填
        [dictM setObject:STS_TOKEN forKey:@"sts_token"]; // 必填
        // 离线语音合成sdk_code取值：精品版为software_nls_tts_offline， 标准版为software_nls_tts_offline_standard
        // 离线语音合成账户和sdk_code可用于唤醒
        // 由创建Appkey时设置
        NSString *sdk_code = @"software_nls_tts_offline_standard";
        [dictM setObject:sdk_code forKey:@"sdk_code"]; // 必填
    } else if (type == get_sts_access_from_server_for_mixed_features) {
        //方法三，适合离在线语音交互服务(强烈推荐):
        //  客户远端服务端使用STS服务获得STS临时凭证，然后下发给移动端侧，详情请查看：
        //    https://help.aliyun.com/document_detail/466615.html 使用其中方案二使用STS获取临时账号。
        //  然后在移动端侧通过AccessToken()获得Token和有效期，用于在线语音交互服务。
        //注意！此处介绍同一个Appkey用于在线和离线功能，用户可创建两个Appkey分别用于在线和离线功能。
        NSString *STS_AK_ID = @"STS.<服务器生成的具有时效性的临时凭证>";
        NSString *STS_AK_SECRET = @"<服务器生成的具有时效性的临时凭证>";
        NSString *STS_TOKEN = @"<服务器生成的具有时效性的临时凭证>";
        NSString *TOKEN = @"<由STS生成的临时访问令牌>";
        TOKEN = [self generateToken:STS_AK_ID withSecret:STS_AK_SECRET withStsToken:STS_TOKEN];
        if (TOKEN == NULL) {
            NSLog(@"generate token failed");
            return;
        }
        [dictM setObject:STS_AK_ID forKey:@"ak_id"]; // 必填
        [dictM setObject:STS_AK_SECRET forKey:@"ak_secret"]; // 必填
        [dictM setObject:STS_TOKEN forKey:@"sts_token"]; // 必填
        [dictM setObject:TOKEN forKey:@"token"]; // 必填
        self.curToken = TOKEN;
        // 离线语音合成sdk_code取值：精品版为software_nls_tts_offline， 标准版为software_nls_tts_offline_standard
        // 离线语音合成账户和sdk_code可用于唤醒
        // 由创建Appkey时设置
        NSString *sdk_code = @"software_nls_tts_offline_standard";
        [dictM setObject:sdk_code forKey:@"sdk_code"]; // 必填
    } else if (type == get_token_from_server_for_online_features) {
        //方法四，仅适合在线语音交互服务(推荐):
        //  客户远端服务端使用Token服务获得Token临时令牌，然后下发给移动端侧，详情请查看：
        //    https://help.aliyun.com/document_detail/466615.html 使用其中方案一获取临时令牌Token
        //  获得Token方法：
        //    https://help.aliyun.com/zh/isi/getting-started/overview-of-obtaining-an-access-token
        NSString *TOKEN = @"<服务器生成的具有时效性的临时凭证>";
        [dictM setObject:TOKEN forKey:@"token"]; // 必填
    } else if (type == get_access_from_server_for_offline_features) {
        //方法五，仅适合离线语音交互服务(不推荐):
        //  客户远端服务端将账号信息ak_id和ak_secret(请加密)下发给移动端侧。
        //注意！账号信息出现在移动端侧，存在泄露风险。
        NSString *AK_ID = @"<一定不可代码中存储和本地明文存储>";
        NSString *AK_SECRET = @"<一定不可代码中存储和本地明文存储>";

        [dictM setObject:AK_ID forKey:@"ak_id"]; // 必填
        [dictM setObject:AK_SECRET forKey:@"ak_secret"]; // 必填
        // 离线语音合成sdk_code取值：精品版为software_nls_tts_offline， 标准版为software_nls_tts_offline_standard
        // 离线语音合成账户和sdk_code可用于唤醒
        // 由创建Appkey时设置
        NSString *sdk_code = @"software_nls_tts_offline_standard";
        [dictM setObject:sdk_code forKey:@"sdk_code"]; // 必填
    } else if (type == get_access_from_server_for_mixed_features) {
        //方法六，适合离在线语音交互服务(不推荐):
        //  客户远端服务端将账号信息ak_id和ak_secret(请加密)下发给移动端侧。
        //  然后在移动端侧通过AccessToken()获得Token和有效期，用于在线语音交互服务。
        //注意！账号信息出现在移动端侧，存在泄露风险。
        NSString *AK_ID = @"<一定不可代码中存储和本地明文存储>";
        NSString *AK_SECRET = @"<一定不可代码中存储和本地明文存储>";
        NSString *TOKEN = @"<生成的临时访问令牌>";
        TOKEN = [self generateToken:AK_ID withSecret:AK_SECRET withStsToken:@""];
        if (TOKEN == NULL) {
            NSLog(@"generate token failed");
            return;
        }
        [dictM setObject:AK_ID forKey:@"ak_id"]; // 必填
        [dictM setObject:AK_SECRET forKey:@"ak_secret"]; // 必填
        [dictM setObject:TOKEN forKey:@"token"]; // 必填
        self.curToken = TOKEN;
        // 离线语音合成sdk_code取值：精品版为software_nls_tts_offline， 标准版为software_nls_tts_offline_standard
        // 离线语音合成账户和sdk_code可用于唤醒
        // 由创建Appkey时设置
        NSString *sdk_code = @"software_nls_tts_offline_standard";
        [dictM setObject:sdk_code forKey:@"sdk_code"]; // 必填
    } else if (type == get_token_in_client_for_online_features) {
        //方法七，仅适合在线语音交互服务(强烈不推荐):
        //  仅仅用于开发和调试
        //注意！账号信息出现在移动端侧，存在泄露风险。
        NSString *TOKEN = @"<移动端写死的访问令牌，仅用于调试>";
        [dictM setObject:TOKEN forKey:@"token"]; // 必填
        self.curToken = TOKEN;
    } else if (type == get_access_in_client_for_offline_features) {
        //方法八，仅适合离线语音交互服务(强烈不推荐):
        //  仅仅用于开发和调试
        //注意！账号信息出现在移动端侧，存在泄露风险。
        NSString *AK_ID = @"<移动端写死的账号信息，仅用于调试>";
        NSString *AK_SECRET = @"<移动端写死的账号信息，仅用于调试>";
        [dictM setObject:AK_ID forKey:@"ak_id"]; // 必填
        [dictM setObject:AK_SECRET forKey:@"ak_secret"]; // 必填
        // 离线语音合成sdk_code取值：精品版为software_nls_tts_offline， 标准版为software_nls_tts_offline_standard
        // 离线语音合成账户和sdk_code可用于唤醒
        // 由创建Appkey时设置
        NSString *sdk_code = @"software_nls_tts_offline_standard";
        [dictM setObject:sdk_code forKey:@"sdk_code"]; // 必填
    } else if (type == get_access_in_client_for_mixed_features) {
        //方法九，适合离在线语音交互服务(强烈不推荐):
        //  仅仅用于开发和调试
        //注意！账号信息出现在移动端侧，存在泄露风险。
        NSString *AK_ID = @"<移动端写死的账号信息，仅用于调试>";
        NSString *AK_SECRET = @"<移动端写死的账号信息，仅用于调试>";
        NSString *TOKEN = @"<生成的临时访问令牌>";
        TOKEN = [self generateToken:AK_ID withSecret:AK_SECRET withStsToken:@""];
        if (TOKEN == NULL) {
            NSLog(@"generate token failed");
            return;
        }
        [dictM setObject:AK_ID forKey:@"ak_id"]; // 必填
        [dictM setObject:AK_SECRET forKey:@"ak_secret"]; // 必填
        [dictM setObject:TOKEN forKey:@"token"]; // 必填
        self.curToken = TOKEN;
        // 离线语音合成sdk_code取值：精品版为software_nls_tts_offline， 标准版为software_nls_tts_offline_standard
        // 离线语音合成账户和sdk_code可用于唤醒
        // 由创建Appkey时设置
        NSString *sdk_code = @"software_nls_tts_offline_standard";
        [dictM setObject:sdk_code forKey:@"sdk_code"]; // 必填
    } else if (type == get_access_in_client_for_online_features) {
        //方法十，适合在线语音交互服务(强烈不推荐):
        //  仅仅用于开发和调试
        //注意！账号信息出现在移动端侧，存在泄露风险。
        NSString *AK_ID = @"<移动端写死的账号信息，仅用于调试>";
        NSString *AK_SECRET = @"<移动端写死的账号信息，仅用于调试>";
        NSString *TOKEN = @"<生成的临时访问令牌>";
        TOKEN = [self generateToken:AK_ID withSecret:AK_SECRET withStsToken:@""];
        if (TOKEN == NULL) {
            NSLog(@"generate token failed");
            return;
        }
        [dictM setObject:AK_ID forKey:@"ak_id"]; // 必填
        [dictM setObject:AK_SECRET forKey:@"ak_secret"]; // 必填
        [dictM setObject:TOKEN forKey:@"token"]; // 必填
        self.curToken = TOKEN;
    } else if (type == get_sts_access_in_client_for_online_features) {
        //方法十一，适合在线语音交互服务(强烈不推荐):
        //  仅仅用于开发和调试
        NSString *STS_AK_ID = @"STS.<移动端写死的账号信息，仅用于调试>";
        NSString *STS_AK_SECRET = @"<移动端写死的账号信息，仅用于调试>";
        NSString *STS_TOKEN = @"<移动端写死的账号信息，仅用于调试>";
        NSString *TOKEN = @"<由STS生成的临时访问令牌>";
        TOKEN = [self generateToken:STS_AK_ID withSecret:STS_AK_SECRET withStsToken:STS_TOKEN];
        if (TOKEN == NULL) {
            NSLog(@"generate token failed");
            return;
        }
        [dictM setObject:STS_AK_ID forKey:@"ak_id"]; // 必填
        [dictM setObject:STS_AK_SECRET forKey:@"ak_secret"]; // 必填
        [dictM setObject:STS_TOKEN forKey:@"sts_token"]; // 必填
        [dictM setObject:TOKEN forKey:@"token"]; // 必填
        self.curToken = TOKEN;
    } else if (type == get_sts_access_in_client_for_offline_features) {
        //方法十二，适合离线语音交互服务(强烈不推荐):
        //  仅仅用于开发和调试
        NSString *STS_AK_ID = @"STS.<移动端写死的账号信息，仅用于调试>";
        NSString *STS_AK_SECRET = @"<移动端写死的账号信息，仅用于调试>";
        NSString *STS_TOKEN = @"<移动端写死的账号信息，仅用于调试>";
        NSString *TOKEN = @"<由STS生成的临时访问令牌>";
        TOKEN = [self generateToken:STS_AK_ID withSecret:STS_AK_SECRET withStsToken:STS_TOKEN];
        if (TOKEN == NULL) {
            NSLog(@"generate token failed");
            return;
        }
        [dictM setObject:STS_AK_ID forKey:@"ak_id"]; // 必填
        [dictM setObject:STS_AK_SECRET forKey:@"ak_secret"]; // 必填
        [dictM setObject:STS_TOKEN forKey:@"sts_token"]; // 必填
        [dictM setObject:TOKEN forKey:@"token"]; // 必填
        self.curToken = TOKEN;
        // 离线语音合成sdk_code取值：精品版为software_nls_tts_offline， 标准版为software_nls_tts_offline_standard
        // 离线语音合成账户和sdk_code可用于唤醒
        // 由创建Appkey时设置
        NSString *sdk_code = @"software_nls_tts_offline_standard";
        [dictM setObject:sdk_code forKey:@"sdk_code"]; // 必填
    } else if (type == get_sts_access_in_client_for_mixed_features) {
        //方法十三，适合离在线语音交互服务(强烈不推荐):
        //  仅仅用于开发和调试
        NSString *STS_AK_ID = @"STS.<移动端写死的账号信息，仅用于调试>";
        NSString *STS_AK_SECRET = @"<移动端写死的账号信息，仅用于调试>";
        NSString *STS_TOKEN = @"<移动端写死的账号信息，仅用于调试>";
        NSString *TOKEN = @"<由STS生成的临时访问令牌>";
        TOKEN = [self generateToken:STS_AK_ID withSecret:STS_AK_SECRET withStsToken:STS_TOKEN];
        if (TOKEN == NULL) {
            NSLog(@"generate token failed");
            return;
        }
        [dictM setObject:STS_AK_ID forKey:@"ak_id"]; // 必填
        [dictM setObject:STS_AK_SECRET forKey:@"ak_secret"]; // 必填
        [dictM setObject:STS_TOKEN forKey:@"sts_token"]; // 必填
        [dictM setObject:TOKEN forKey:@"token"]; // 必填
        self.curToken = TOKEN;
        // 离线语音合成sdk_code取值：精品版为software_nls_tts_offline， 标准版为software_nls_tts_offline_standard
        // 离线语音合成账户和sdk_code可用于唤醒
        // 由创建Appkey时设置
        NSString *sdk_code = @"software_nls_tts_offline_standard";
        [dictM setObject:sdk_code forKey:@"sdk_code"]; // 必填
    }
}

-(NSString*)generateToken:(NSString*)accessKey withSecret:(NSString*)accessSecret withStsToken:(NSString*)stsToken {
    AccessToken *accessToken = [[AccessToken alloc]initWithAccessKeyId:accessKey andAccessSecret:accessSecret andSecurityToken:stsToken];
    [accessToken apply];
    long expire_time = [accessToken expireTime];
    NSLog(@"Token expire time is %ld", expire_time);
    self.curTokenExpiredTime = expire_time;
    return [accessToken token];
}

- (void)refreshTokenIfNeed:(NSMutableDictionary *)json distanceExpireTime:(long)distanceExpireTime {
    if (self.curAppkey.length > 0 && self.curToken.length > 0 && self.curTokenExpiredTime > 0) {
        long millis = (long)([[NSDate date] timeIntervalSince1970] * 1000);
        long unixTimestampInSeconds = millis / 1000;
        
        if (self.curTokenExpiredTime - distanceExpireTime < unixTimestampInSeconds) {
            NSString *oldToken = self.curToken;
            long oldExpireTime = self.curTokenExpiredTime;
            
            NSMutableDictionary *ticketJsonDict = [NSMutableDictionary dictionary];
            [self getTicket:ticketJsonDict Type:self.curTokenTicketType];
            if ([ticketJsonDict objectForKey:@"token"] != nil) {
                self.curToken = [ticketJsonDict objectForKey:@"token"];
                if ([self.curToken length] == 0) {
                    TLog(@"The 'token' key exists but the value is empty.");
                }
                [json setObject:self.curToken forKey:@"token"];
            } else {
                TLog(@"The 'token' key does not exist.");
            }
            if ([ticketJsonDict objectForKey:@"app_key"] != nil) {
                self.curAppkey = [ticketJsonDict objectForKey:@"app_key"];
                if ([self.curAppkey length] == 0) {
                    TLog(@"The 'app_key' key exists but the value is empty.");
                }
                [json setObject:self.curAppkey forKey:@"app_key"];
            } else {
                TLog(@"The 'app_key' key does not exist.");
            }
            
            NSString *newToken = self.curToken;
            long newExpireTime = self.curTokenExpiredTime;
            
            NSLog(@"Refresh old token(%@ : %ld) to (%@ : %ld).", oldToken, oldExpireTime, newToken, newExpireTime);
        }
    }
}

-(NSString*) getDirectIp {
    const int MAX_HOST_IP_LENGTH = 16;
    struct hostent *remoteHostEnt = gethostbyname("nls-gateway-inner.aliyuncs.com");
    if(remoteHostEnt == NULL) {
        NSLog(@"demo get host failed!");
    }
    struct in_addr *remoteInAddr = (struct in_addr *) remoteHostEnt->h_addr_list[0];
    //ip = inet_ntoa(*remoteInAddr);
    char ip_[MAX_HOST_IP_LENGTH];
    inet_ntop(AF_INET, (void *)remoteInAddr, ip_, MAX_HOST_IP_LENGTH);
    NSString *ip=[NSString stringWithUTF8String:ip_];
    return ip;
}

-(NSString*) getGuideWithError:(int)errorCode withError:(NSString*)errMesg withStatus:(NSString*)status {
    NSString * str = errMesg;
    switch (errorCode) {
        case 140001:
            str = @" 错误信息: 引擎未创建, 请检查是否成功初始化, 详情可查看运行日志.";
            break;
        case 140008:
            str = @" 错误信息: 鉴权失败, 请关注日志中详细失败原因.";
            break;
        case 140011:
            str = @" 错误信息: 当前方法调用不符合当前状态, 比如在未初始化情况下调用pause接口.";
            break;
        case 140013:
            str = @" 错误信息: 当前方法调用不符合当前状态, 比如在未初始化情况下调用pause/release等接口.";
            break;
        case 140900:
            str = @" 错误信息: tts引擎初始化失败, 请检查资源路径和资源文件是否正确.";
            break;
        case 140901:
            str = @" 错误信息: tts引擎初始化失败, 请检查使用的SDK是否支持离线语音合成功能.";
            break;
        case 140903:
            str = @" 错误信息: tts引擎任务创建失败, 请检查资源路径和资源文件是否正确.";
            break;
        case 140908:
            str = @" 错误信息: 发音人资源无法获得正确采样率, 请检查发音人资源是否正确.";
            break;
        case 140910:
            str = @" 错误信息: 发音人资源路径无效, 请检查发音人资源文件路径是否正确.";
            break;
        case 144002:
            str = @" 错误信息: 若发生于语音合成, 可能为传入文本超过16KB. 可升级到最新版本, 具体查看日志确认.";
            break;
        case 144003:
            str = @" 错误信息: token过期或无效, 请检查token是否有效.";
            break;
        case 144004:
            str = @" 错误信息: 语音合成超时, 具体查看日志确认.";
            break;
        case 144006:
            str = @" 错误信息: 云端返回未分类错误, 请看详细的错误信息.";
            break;
        case 144103:
            str = @" 错误信息: 设置参数无效, 请参考接口文档检查参数是否正确, 也可通过task_id咨询客服.";
            break;
        case 144500:
            str = @" 错误信息: 流式TTS状态错误, 可能是在停止状态调用接口.";
            break;
        case 170008:
            str = @" 错误信息: 鉴权成功, 但是存储鉴权信息的文件路径不存在或无权限.";
            break;
        case 170806:
            str = @" 错误信息: 请设置SecurityToken.";
            break;
        case 170807:
            str = @" 错误信息: SecurityToken过期或无效, 请检查SecurityToken是否有效.";
            break;
        case 240002:
            str = @" 错误信息: 设置的参数不正确, 比如设置json参数格式不对, 设置的文件无效等.";
            break;
        case 240005:
            if ([status isEqualToString:@"init"]) {
                str = @" 错误信息: 请检查appkey、akId、akSecret、url等初始化参数是否无效或空.";
            } else {
                str = @" 错误信息: 传入参数无效, 请检查参数正确性.";
            }
            break;
        case 240008:
            str = @" 错误信息: SDK内部核心引擎未成功初始化.";
            break;
        case 240011:
            str = @" 错误信息: SDK未成功初始化.";
            break;
        case 240040:
            str = @" 错误信息: 本地引擎初始化失败，可能是资源文件(如kws.bin)损坏.";
            break;
        case 240052:
            str = @" 错误信息: 2s未传入音频数据，请检查录音相关代码、权限或录音模块是否被其他应用占用.";
            break;
        case 240063:
            str = @" 错误信息: SSL错误，可能为SSL建连失败。比如token无效或者过期，或SSL证书校验失败(可升级到最新版)等等，具体查日志确认.";
            break;
        case 240068:
            str = @" 错误信息: 403 Forbidden, token无效或者过期.";
            break;
        case 240070:
            str = @" 错误信息: 鉴权失败, 请查看日志确定具体问题, 特别是关注日志 E/iDST::ErrMgr: errcode=.";
            break;
        case 240072:
            str = @" 错误信息: 录音文件识别传入的录音文件不存在.";
            break;
        case 240073:
            str = @" 错误信息: 录音文件识别传入的参数错误, 比如audio_address不存在或file_path不存在或其他参数错误.";
            break;
        case 10000016:
            if ([status rangeOfString:@"403 Forbidden"].location != NSNotFound) {
                str = @" 错误信息: 流式语音合成未成功连接服务, 请检查设置的账号临时凭证.";
            } else if ([status rangeOfString:@"404 Forbidden"].location != NSNotFound) {
                str = @" 错误信息: 流式语音合成未成功连接服务, 请检查设置的服务地址URL.";
            } else {
                str = @" 错误信息: 流式语音合成未成功连接服务, 请检查设置的参数及服务地址.";
            }
            break;
        case 40000004:
            str = @" 错误信息: 长时间未收到指令或音频.";
            break;
        case 40000010:
            if ([errMesg rangeOfString:@"FREE_TRIAL_EXPIRED"].location != NSNotFound) {
                str = @" 错误信息: 此账号试用期已过, 请开通商用版或检查账号权限.";
            } else {
                str = errMesg;
            }
            break;
        case 41010105:
            str = @" 错误信息: 长时间未收到人声, 触发静音超时.";
            break;
        case 999999:
            str = @" 错误信息: 库加载失败, 可能是库不支持当前服务, 或库加载时崩溃, 可详细查看日志判断.";
            break;
        default:
            str = errMesg;
    }

    return str;
}

@end
